@@ -25,32 +25,37 @@ pub fn generate(program: &Program) -> proc_macro2::TokenStream {
 
                 match ix {
                     anchor_lang::idl::IdlInstruction::Create { data_len } => {
+                        let mut bumps = std::collections::BTreeMap::new();
                         let mut accounts =
-                            anchor_lang::idl::IdlCreateAccounts::try_accounts(program_id, &mut accounts, &[])?;
+                            anchor_lang::idl::IdlCreateAccounts::try_accounts(program_id, &mut accounts, &[], &mut bumps)?;
                         __idl_create_account(program_id, &mut accounts, data_len)?;
                         accounts.exit(program_id)?;
                     },
                     anchor_lang::idl::IdlInstruction::CreateBuffer => {
+                        let mut bumps = std::collections::BTreeMap::new();
                         let mut accounts =
-                            anchor_lang::idl::IdlCreateBuffer::try_accounts(program_id, &mut accounts, &[])?;
+                            anchor_lang::idl::IdlCreateBuffer::try_accounts(program_id, &mut accounts, &[], &mut bumps)?;
                         __idl_create_buffer(program_id, &mut accounts)?;
                         accounts.exit(program_id)?;
                     },
                     anchor_lang::idl::IdlInstruction::Write { data } => {
+                        let mut bumps = std::collections::BTreeMap::new();
                         let mut accounts =
-                            anchor_lang::idl::IdlAccounts::try_accounts(program_id, &mut accounts, &[])?;
+                            anchor_lang::idl::IdlAccounts::try_accounts(program_id, &mut accounts, &[], &mut bumps)?;
                         __idl_write(program_id, &mut accounts, data)?;
                         accounts.exit(program_id)?;
                     },
                     anchor_lang::idl::IdlInstruction::SetAuthority { new_authority } => {
+                        let mut bumps = std::collections::BTreeMap::new();
                         let mut accounts =
-                            anchor_lang::idl::IdlAccounts::try_accounts(program_id, &mut accounts, &[])?;
+                            anchor_lang::idl::IdlAccounts::try_accounts(program_id, &mut accounts, &[], &mut bumps)?;
                         __idl_set_authority(program_id, &mut accounts, new_authority)?;
                         accounts.exit(program_id)?;
                     },
                     anchor_lang::idl::IdlInstruction::SetBuffer => {
+                        let mut bumps = std::collections::BTreeMap::new();
                         let mut accounts =
-                            anchor_lang::idl::IdlSetBuffer::try_accounts(program_id, &mut accounts, &[])?;
+                            anchor_lang::idl::IdlSetBuffer::try_accounts(program_id, &mut accounts, &[], &mut bumps)?;
                         __idl_set_buffer(program_id, &mut accounts)?;
                         accounts.exit(program_id)?;
                     },
@@ -210,10 +215,14 @@ pub fn generate(program: &Program) -> proc_macro2::TokenStream {
                                 .map_err(|_| anchor_lang::__private::ErrorCode::InstructionDidNotDeserialize)?;
                             let instruction::state::#variant_arm = ix;
 
+                            let mut __bumps = std::collections::BTreeMap::new();
+
                             // Deserialize accounts.
                             let mut remaining_accounts: &[AccountInfo] = accounts;
-                            let ctor_accounts = anchor_lang::__private::Ctor::try_accounts(program_id, &mut remaining_accounts, &[])?;
-                            let mut ctor_user_def_accounts = #anchor_ident::try_accounts(program_id, &mut remaining_accounts, ix_data)?;
+                            let ctor_accounts =
+                            anchor_lang::__private::Ctor::try_accounts(program_id, &mut remaining_accounts, &[], &mut __bumps)?;
+                            let mut ctor_user_def_accounts =
+                            #anchor_ident::try_accounts(program_id, &mut remaining_accounts, ix_data, &mut __bumps)?;
 
                             // Create the solana account for the ctor data.
                             let from = ctor_accounts.from.key;
@@ -258,6 +267,7 @@ pub fn generate(program: &Program) -> proc_macro2::TokenStream {
                                         program_id,
                                         &mut ctor_user_def_accounts,
                                         remaining_accounts,
+                                        __bumps,
                                     ),
                                     #(#ctor_untyped_args),*
                                 )?;
@@ -284,10 +294,14 @@ pub fn generate(program: &Program) -> proc_macro2::TokenStream {
                                 .map_err(|_| anchor_lang::__private::ErrorCode::InstructionDidNotDeserialize)?;
                             let instruction::state::#variant_arm = ix;
 
+                            let mut __bumps = std::collections::BTreeMap::new();
+
                             // Deserialize accounts.
                             let mut remaining_accounts: &[AccountInfo] = accounts;
-                            let ctor_accounts = anchor_lang::__private::Ctor::try_accounts(program_id, &mut remaining_accounts, &[])?;
-                            let mut ctor_user_def_accounts = #anchor_ident::try_accounts(program_id, &mut remaining_accounts, ix_data)?;
+                            let ctor_accounts =
+                            anchor_lang::__private::Ctor::try_accounts(program_id, &mut remaining_accounts, &[], &mut __bumps)?;
+                            let mut ctor_user_def_accounts =
+                            #anchor_ident::try_accounts(program_id, &mut remaining_accounts, ix_data, &mut __bumps)?;
 
                             // Invoke the ctor.
                             let instance = #mod_name::#name::new(
@@ -295,6 +309,7 @@ pub fn generate(program: &Program) -> proc_macro2::TokenStream {
                                     program_id,
                                     &mut ctor_user_def_accounts,
                                     remaining_accounts,
+                                    __bumps,
                                 ),
                                 #(#ctor_untyped_args),*
                             )?;
@@ -387,20 +402,30 @@ pub fn generate(program: &Program) -> proc_macro2::TokenStream {
                                         .map_err(|_| anchor_lang::__private::ErrorCode::InstructionDidNotDeserialize)?;
                                     let instruction::state::#variant_arm = ix;
 
+                                    // Bump collector.
+                                    let mut __bumps = std::collections::BTreeMap::new();
+
                                     // Load state.
                                     let mut remaining_accounts: &[AccountInfo] = accounts;
                                     if remaining_accounts.is_empty() {
                                         return Err(anchor_lang::__private::ErrorCode::AccountNotEnoughKeys.into());
                                     }
-                                    let loader: anchor_lang::accounts::loader::Loader<#mod_name::#name> = anchor_lang::accounts::loader::Loader::try_accounts(program_id, &mut remaining_accounts, &[])?;
+                                    let loader: anchor_lang::accounts::loader::Loader<#mod_name::#name> = anchor_lang::accounts::loader::Loader::try_accounts(program_id, &mut remaining_accounts, &[], &mut __bumps)?;
 
                                     // Deserialize accounts.
                                     let mut accounts = #anchor_ident::try_accounts(
                                         program_id,
                                         &mut remaining_accounts,
                                         ix_data,
+                                        &mut __bumps,
                                     )?;
-                                    let ctx = anchor_lang::context::Context::new(program_id, &mut accounts, remaining_accounts);
+                                    let ctx =
+                                        anchor_lang::context::Context::new(
+                                            program_id,
+                                            &mut accounts,
+                                            remaining_accounts,
+                                            __bumps,
+                                        );
 
                                     // Execute user defined function.
                                     {
@@ -433,20 +458,35 @@ pub fn generate(program: &Program) -> proc_macro2::TokenStream {
                                         .map_err(|_| anchor_lang::__private::ErrorCode::InstructionDidNotDeserialize)?;
                                     let instruction::state::#variant_arm = ix;
 
+                                    // Bump collector.
+                                    let mut __bumps = std::collections::BTreeMap::new();
+
                                     // Load state.
                                     let mut remaining_accounts: &[AccountInfo] = accounts;
                                     if remaining_accounts.is_empty() {
                                         return Err(anchor_lang::__private::ErrorCode::AccountNotEnoughKeys.into());
                                     }
-                                    let mut state: anchor_lang::accounts::state::ProgramState<#state_ty> = anchor_lang::accounts::state::ProgramState::try_accounts(program_id, &mut remaining_accounts, &[])?;
+                                    let mut state: anchor_lang::accounts::state::ProgramState<#state_ty> = anchor_lang::accounts::state::ProgramState::try_accounts(
+                                        program_id,
+                                        &mut remaining_accounts,
+                                        &[],
+                                        &mut __bumps,
+                                    )?;
 
                                     // Deserialize accounts.
                                     let mut accounts = #anchor_ident::try_accounts(
                                         program_id,
                                         &mut remaining_accounts,
                                         ix_data,
+                                        &mut __bumps,
                                     )?;
-                                    let ctx = anchor_lang::context::Context::new(program_id, &mut accounts, remaining_accounts);
+                                    let ctx =
+                                        anchor_lang::context::Context::new(
+                                            program_id,
+                                            &mut accounts,
+                                            remaining_accounts,
+                                            __bumps
+                                        );
 
                                     // Execute user defined function.
                                     state.#ix_method_name(
@@ -488,7 +528,7 @@ pub fn generate(program: &Program) -> proc_macro2::TokenStream {
                             .map(|ix| {
                                 // Easy to implement. Just need to write a test.
                                 // Feel free to open a PR.
-                                assert!(!state.is_zero_copy, "Trait implementations not yet implemented for zero copy state structs. Please file an issue.");                                
+                                assert!(!state.is_zero_copy, "Trait implementations not yet implemented for zero copy state structs. Please file an issue.");
 
                                 let ix_arg_names: Vec<&syn::Ident> =
                                     ix.args.iter().map(|arg| &arg.name).collect();
@@ -546,20 +586,35 @@ pub fn generate(program: &Program) -> proc_macro2::TokenStream {
                                             // Deserialize instruction.
                                             #deserialize_instruction
 
+                                            // Bump collector.
+                                            let mut __bumps = std::collections::BTreeMap::new();
+
                                             // Deserialize the program state account.
                                             let mut remaining_accounts: &[AccountInfo] = accounts;
                                             if remaining_accounts.is_empty() {
                                                 return Err(anchor_lang::__private::ErrorCode::AccountNotEnoughKeys.into());
                                             }
-                                            let mut state: anchor_lang::accounts::state::ProgramState<#state_ty> = anchor_lang::accounts::state::ProgramState::try_accounts(program_id, &mut remaining_accounts, &[])?;
+                                            let mut state: anchor_lang::accounts::state::ProgramState<#state_ty> = anchor_lang::accounts::state::ProgramState::try_accounts(
+                                                program_id,
+                                                &mut remaining_accounts,
+                                                &[],
+                                                &mut __bumps,
+                                            )?;
 
                                             // Deserialize accounts.
                                             let mut accounts = #anchor_ident::try_accounts(
                                                 program_id,
                                                 &mut remaining_accounts,
                                                 ix_data,
+                                                &mut __bumps,
                                             )?;
-                                            let ctx = anchor_lang::context::Context::new(program_id, &mut accounts, remaining_accounts);
+                                            let ctx =
+                                                anchor_lang::context::Context::new(
+                                                    program_id,
+                                                    &mut accounts,
+                                                    remaining_accounts,
+                                                    __bumps,
+                                                );
 
                                             // Execute user defined function.
                                             state.#ix_method_name(
@@ -593,17 +648,26 @@ pub fn generate(program: &Program) -> proc_macro2::TokenStream {
                                             // Deserialize instruction.
                                             #deserialize_instruction
 
+                                            // Bump collector.
+                                            let mut __bumps = std::collections::BTreeMap::new();
+
                                             // Deserialize accounts.
                                             let mut remaining_accounts: &[AccountInfo] = accounts;
                                             let mut accounts = #anchor_ident::try_accounts(
                                                 program_id,
                                                 &mut remaining_accounts,
                                                 ix_data,
+                                                &mut __bumps,
                                             )?;
 
                                             // Execute user defined function.
                                             #state_name::#ix_method_name(
-                                                anchor_lang::context::Context::new(program_id, &mut accounts, remaining_accounts),
+                                                anchor_lang::context::Context::new(
+                                                    program_id,
+                                                    &mut accounts,
+                                                    remaining_accounts,
+                                                    __bumps
+                                                ),
                                                 #(#ix_arg_names),*
                                             )?;
 
@@ -644,17 +708,26 @@ pub fn generate(program: &Program) -> proc_macro2::TokenStream {
                         .map_err(|_| anchor_lang::__private::ErrorCode::InstructionDidNotDeserialize)?;
                     let instruction::#variant_arm = ix;
 
+                    // Bump collector.
+                    let mut __bumps = std::collections::BTreeMap::new();
+
                     // Deserialize accounts.
                     let mut remaining_accounts: &[AccountInfo] = accounts;
                     let mut accounts = #anchor::try_accounts(
                         program_id,
                         &mut remaining_accounts,
                         ix_data,
+                        &mut __bumps,
                     )?;
 
                     // Invoke user defined handler.
                     #program_name::#ix_method_name(
-                        anchor_lang::context::Context::new(program_id, &mut accounts, remaining_accounts),
+                        anchor_lang::context::Context::new(
+                            program_id,
+                            &mut accounts,
+                            remaining_accounts,
+                            __bumps,
+                        ),
                         #(#ix_arg_names),*
                     )?;
 
